// test.c
/*
 * PM - 2017
 * lab0.c
 */
#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>


int step1 = 0;
int step2 = 0;
int pulse_width = 2000;

void my_delay(int period) {
	while (period > 0) {
		_delay_ms(1);
		period--;
	}
}
void rotateClockwise(volatile uint8_t *port, int pin0, int pin1, int pin2, int pin3, int period, int steps) {

	for(int i = 0; i < steps; i++) {
		(*port) = 0;
		(*port) |= (1 << pin0);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin1);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin2);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin3);
		my_delay(period);
	}
}
void rotateAntiClockwise(volatile uint8_t *port, int pin0, int pin1, int pin2, int pin3, int period, int steps) {

	for(int i = 0; i < steps; i++) {
		(*port) = 0;
		(*port) |= (1 << pin3);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin2);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin1);
		my_delay(period);

		(*port) = 0;
		(*port) |= (1 << pin0);
		my_delay(period);
	}
}
void set_angle(int angle) {
	OCR1A = 61 + (angle * 63) / 180; // 124 - 61
}
void set_speed(int procent) {
	OCR0A = procent * 255 / 100;
}
void init_timer0() {

	TCCR0A |= (3 << WGM00) | (1 << COM0A1);
	TCCR0B |= (1 << CS00);
	OCR0A = 0;
}
void init_pen() {
	DDRD |= (1 << PD5);
	TCCR1A |= (1 << WGM10);
	TCCR1B |= (1 << WGM12);
	TCCR1A |= (1 << COM1A1);
	TCCR1B |= (1 << CS12) | (1 << CS10);
	OCR1A =  30;
 	_delay_ms(1000);
}
void raise_pen() {

 	OCR1A =  30;
 	_delay_ms(1000);
}
void lower_pen() {

 	OCR1A =  15;
 	_delay_ms(1000);
}

void init_stepper1() {

	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	DDRA |= (1 << PA3);
}
void init_stepper2() {

	DDRA |= (1 << PA4);
	DDRA |= (1 << PA5);
	DDRA |= (1 << PA6);
	DDRA |= (1 << PA7);
}
void init_blink() {

	DDRD |= (1 << PD7);
}
/*
 * Functie de initializare a controllerului USART
 */
void USART0_init()
{
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 102;

    /* porneste transmitatorul */
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, paritate para */
    UCSR0C &= ~(1<<USBS0);
    UCSR0C |= (2<<UCSZ00);
    UCSR0C |= (1<<UPM01);
}
/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}
/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 * @param stream - nefolosit
 *
 * @return - intotdeauna returneaza 0
 */
void read_input() {


}
/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART0_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}
void USART0_print(const char *data)
{
    while(*data != '\0')
        USART0_transmit(*data++);
}


int main(void) {

/*
	Stepper 1 : 
		PA0 - IN1
		PA1 - IN2
		PA2 - IN3
		PA3 - IN4

	Stepper 2 :
		PA4 - IN1
		PA5 - IN2
		PA6 - IN3
		PA7 - IN4

	Servomotor:
		PD5
*/

	int period = 50;

	/* Make sure the pen is above the paper */
	init_pen();

	/* Set stepper motors */
	init_stepper1();
	init_stepper2();

	/* Set blinking LED for testing */
	init_blink();

	/* Check for input letter */
	USART0_init();
	char command;
	char buffer[100];
	
	while(1) {
		rotateClockwise(&PORTA, PA0, PA1, PA2, PA3, period, 50);
		rotateAntiClockwise(&PORTA, PA4, PA5, PA6, PA7, period, 50);
	}

	while(1) {
		USART0_transmit('a');
		command = USART0_receive();
		USART0_transmit('p');
		switch(command) {
			case 'x': {
				/* move stepper on the Ox axis */
				sprintf(buffer, "Introduceti valoarea de pe axa Ox:\n[-10 < value < 10]\n");
				USART0_print(buffer);
				const char c = USART0_receive();
				int pos = atoi(&c);
				if (pos > 0)
					rotateClockwise(&PORTA, PA0, PA1, PA2, PA3, period, pos);
				else
					rotateAntiClockwise(&PORTA, PA0, PA1, PA2, PA3, period, pos);
				break;
			}
			case 'y': {
				/* move stepper on the Oy axis */
				/* move stepper on the Ox axis */
				USART0_print("Introduceti valoarea de pe axa Oy:\n[-10 < value < 10]\n");
				const char c = USART0_receive();
				int pos = atoi(&c);
				if (pos > 0)
					rotateClockwise(&PORTA, PA4, PA5, PA6, PA7, period, pos);
				else
					rotateAntiClockwise(&PORTA, PA4, PA5, PA6, PA7, period, pos);
				break;
			}
			case 'u': {
				/* raise pen */
				USART0_print("Raising the pen\n");
				raise_pen();
				break;
			}
			case 'd': {
				/* lower pen */
				USART0_print("Lowering the pen\n");
				lower_pen();
				break;
			}
			default:
				break;
		}
		PORTD ^= (1 << PD7);
		_delay_ms(period);


	}

	return 0;
}

